'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fiber = require('@react-three/fiber');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

var lowerCaseFirstLetter = function lowerCaseFirstLetter(str) {
  return "" + str.charAt(0).toLowerCase() + str.slice(1);
};
var toEventHandlerName = function toEventHandlerName(eventName) {
  return "on" + eventName.charAt(0).toUpperCase() + eventName.slice(1);
};

var treeObjectFactory = function treeObjectFactory(type, props, children) {
  return {
    type: type,
    props: props,
    children: children
  };
};

var toTreeBranch = function toTreeBranch(obj) {
  return obj.map(function (child) {
    return treeObjectFactory(lowerCaseFirstLetter(child.type || child.constructor.name), _extends({}, child.__r3f.memoizedProps), toTreeBranch([].concat(child.children || [], child.__r3f.objects)));
  });
};

var toTree = function toTree(root) {
  return root.children.map(function (obj) {
    return treeObjectFactory(lowerCaseFirstLetter(obj.type), _extends({}, obj.__r3f.memoizedProps), toTreeBranch([].concat(obj.children, obj.__r3f.objects)));
  });
};

var graphObjectFactory = function graphObjectFactory(type, name, children) {
  return {
    type: type,
    name: name,
    children: children
  };
};

var toGraph = function toGraph(object) {
  return object.children.map(function (child) {
    return graphObjectFactory(child.type, child.name || '', toGraph(child));
  });
};

var is = {
  obj: function obj(a) {
    return a === Object(a) && !is.arr(a) && typeof a !== 'function';
  },
  fun: function fun(a) {
    return typeof a === 'function';
  },
  str: function str(a) {
    return typeof a === 'string';
  },
  num: function num(a) {
    return typeof a === 'number';
  },
  und: function und(a) {
    return a === void 0;
  },
  arr: function arr(a) {
    return Array.isArray(a);
  },
  equ: function equ(a, b) {
    // Wrong type or one of the two undefined, doesn't match
    if (typeof a !== typeof b || !!a !== !!b) return false; // Atomic, just compare a against b

    if (is.str(a) || is.num(a) || is.obj(a)) return a === b; // Array, shallow compare first to see if it's a match

    if (is.arr(a) && a == b) return true; // Last resort, go through keys

    var i;

    for (i in a) {
      if (!(i in b)) return false;
    }

    for (i in b) {
      if (a[i] !== b[i]) return false;
    }

    return is.und(i) ? a === b : true;
  }
};

var createCanvas = function createCanvas(_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      beforeReturn = _ref.beforeReturn,
      _ref$width = _ref.width,
      width = _ref$width === void 0 ? 1280 : _ref$width,
      _ref$height = _ref.height,
      height = _ref$height === void 0 ? 800 : _ref$height;

  var canvas = document.createElement('canvas');
  canvas.height = height;
  canvas.width = width;

  canvas.addEventListener = function () {};

  canvas.removeEventListener = function () {};

  if (beforeReturn) {
    beforeReturn(canvas);
  }

  return canvas;
};

var functions = ['attachShader', 'bindAttribLocation', 'bindBuffer', 'bindFramebuffer', 'bindRenderbuffer', 'bindTexture', 'blendColor', 'blendEquation', 'blendEquationSeparate', 'blendFunc', 'blendFuncSeparate', 'bufferData', 'bufferSubData', 'checkFramebufferStatus', 'clear', 'clearColor', 'clearDepth', 'clearStencil', 'colorMask', 'compileShader', 'compressedTexImage2D', 'compressedTexSubImage2D', 'copyTexImage2D', 'copyTexSubImage2D', 'createBuffer', 'createFramebuffer', 'createProgram', 'createRenderbuffer', 'createShader', 'createTexture', 'cullFace', 'deleteBuffer', 'deleteFramebuffer', 'deleteProgram', 'deleteRenderbuffer', 'deleteShader', 'deleteTexture', 'depthFunc', 'depthMask', 'depthRange', 'detachShader', 'disable', 'disableVertexAttribArray', 'drawArrays', 'drawElements', 'enable', 'enableVertexAttribArray', 'finish', 'flush', 'framebufferRenderbuffer', 'framebufferTexture2D', 'frontFace', 'generateMipmap', 'getActiveAttrib', 'getActiveUniform', 'getAttachedShaders', 'getAttribLocation', 'getBufferParameter', 'getContextAttributes', 'getError', 'getFramebufferAttachmentParameter', 'getProgramParameter', 'getRenderbufferParameter', 'getShaderParameter', 'getShaderSource', 'getSupportedExtensions', 'getTexParameter', 'getUniform', 'getUniformLocation', 'getVertexAttrib', 'getVertexAttribOffset', 'hint', 'isBuffer', 'isContextLost', 'isEnabled', 'isFramebuffer', 'isProgram', 'isRenderbuffer', 'isShader', 'isTexture', 'lineWidth', 'linkProgram', 'pixelStorei', 'polygonOffset', 'readPixels', 'renderbufferStorage', 'sampleCoverage', 'scissor', 'setPixelRatio', 'setSize', 'shaderSource', 'stencilFunc', 'stencilFuncSeparate', 'stencilMask', 'stencilMaskSeparate', 'stencilOp', 'stencilOpSeparate', 'texParameterf', 'texParameteri', 'texImage2D', 'texSubImage2D', 'uniform1f', 'uniform1fv', 'uniform1i', 'uniform1iv', 'uniform2f', 'uniform2fv', 'uniform2i', 'uniform2iv', 'uniform3f', 'uniform3fv', 'uniform3i', 'uniform3iv', 'uniform4f', 'uniform4fv', 'uniform4i', 'uniform4iv', 'uniformMatrix2fv', 'uniformMatrix3fv', 'uniformMatrix4fv', 'useProgram', 'validateProgram', 'vertexAttrib1f', 'vertexAttrib1fv', 'vertexAttrib2f', 'vertexAttrib2fv', 'vertexAttrib3f', 'vertexAttrib3fv', 'vertexAttrib4f', 'vertexAttrib4fv', 'vertexAttribPointer', 'viewport'];
var enums = {
  DEPTH_BUFFER_BIT: 256,
  STENCIL_BUFFER_BIT: 1024,
  COLOR_BUFFER_BIT: 16384,
  POINTS: 0,
  LINES: 1,
  LINE_LOOP: 2,
  LINE_STRIP: 3,
  TRIANGLES: 4,
  TRIANGLE_STRIP: 5,
  TRIANGLE_FAN: 6,
  ZERO: 0,
  ONE: 1,
  SRC_COLOR: 768,
  ONE_MINUS_SRC_COLOR: 769,
  SRC_ALPHA: 770,
  ONE_MINUS_SRC_ALPHA: 771,
  DST_ALPHA: 772,
  ONE_MINUS_DST_ALPHA: 773,
  DST_COLOR: 774,
  ONE_MINUS_DST_COLOR: 775,
  SRC_ALPHA_SATURATE: 776,
  FUNC_ADD: 32774,
  BLEND_EQUATION: 32777,
  BLEND_EQUATION_RGB: 32777,
  BLEND_EQUATION_ALPHA: 34877,
  FUNC_SUBTRACT: 32778,
  FUNC_REVERSE_SUBTRACT: 32779,
  BLEND_DST_RGB: 32968,
  BLEND_SRC_RGB: 32969,
  BLEND_DST_ALPHA: 32970,
  BLEND_SRC_ALPHA: 32971,
  CONSTANT_COLOR: 32769,
  ONE_MINUS_CONSTANT_COLOR: 32770,
  CONSTANT_ALPHA: 32771,
  ONE_MINUS_CONSTANT_ALPHA: 32772,
  BLEND_COLOR: 32773,
  ARRAY_BUFFER: 34962,
  ELEMENT_ARRAY_BUFFER: 34963,
  ARRAY_BUFFER_BINDING: 34964,
  ELEMENT_ARRAY_BUFFER_BINDING: 34965,
  STREAM_DRAW: 35040,
  STATIC_DRAW: 35044,
  DYNAMIC_DRAW: 35048,
  BUFFER_SIZE: 34660,
  BUFFER_USAGE: 34661,
  CURRENT_VERTEX_ATTRIB: 34342,
  FRONT: 1028,
  BACK: 1029,
  FRONT_AND_BACK: 1032,
  TEXTURE_2D: 3553,
  CULL_FACE: 2884,
  BLEND: 3042,
  DITHER: 3024,
  STENCIL_TEST: 2960,
  DEPTH_TEST: 2929,
  SCISSOR_TEST: 3089,
  POLYGON_OFFSET_FILL: 32823,
  SAMPLE_ALPHA_TO_COVERAGE: 32926,
  SAMPLE_COVERAGE: 32928,
  NO_ERROR: 0,
  INVALID_ENUM: 1280,
  INVALID_VALUE: 1281,
  INVALID_OPERATION: 1282,
  OUT_OF_MEMORY: 1285,
  CW: 2304,
  CCW: 2305,
  LINE_WIDTH: 2849,
  ALIASED_POINT_SIZE_RANGE: 33901,
  ALIASED_LINE_WIDTH_RANGE: 33902,
  CULL_FACE_MODE: 2885,
  FRONT_FACE: 2886,
  DEPTH_RANGE: 2928,
  DEPTH_WRITEMASK: 2930,
  DEPTH_CLEAR_VALUE: 2931,
  DEPTH_FUNC: 2932,
  STENCIL_CLEAR_VALUE: 2961,
  STENCIL_FUNC: 2962,
  STENCIL_FAIL: 2964,
  STENCIL_PASS_DEPTH_FAIL: 2965,
  STENCIL_PASS_DEPTH_PASS: 2966,
  STENCIL_REF: 2967,
  STENCIL_VALUE_MASK: 2963,
  STENCIL_WRITEMASK: 2968,
  STENCIL_BACK_FUNC: 34816,
  STENCIL_BACK_FAIL: 34817,
  STENCIL_BACK_PASS_DEPTH_FAIL: 34818,
  STENCIL_BACK_PASS_DEPTH_PASS: 34819,
  STENCIL_BACK_REF: 36003,
  STENCIL_BACK_VALUE_MASK: 36004,
  STENCIL_BACK_WRITEMASK: 36005,
  VIEWPORT: 2978,
  SCISSOR_BOX: 3088,
  COLOR_CLEAR_VALUE: 3106,
  COLOR_WRITEMASK: 3107,
  UNPACK_ALIGNMENT: 3317,
  PACK_ALIGNMENT: 3333,
  MAX_TEXTURE_SIZE: 3379,
  MAX_VIEWPORT_DIMS: 3386,
  SUBPIXEL_BITS: 3408,
  RED_BITS: 3410,
  GREEN_BITS: 3411,
  BLUE_BITS: 3412,
  ALPHA_BITS: 3413,
  DEPTH_BITS: 3414,
  STENCIL_BITS: 3415,
  POLYGON_OFFSET_UNITS: 10752,
  POLYGON_OFFSET_FACTOR: 32824,
  TEXTURE_BINDING_2D: 32873,
  SAMPLE_BUFFERS: 32936,
  SAMPLES: 32937,
  SAMPLE_COVERAGE_VALUE: 32938,
  SAMPLE_COVERAGE_INVERT: 32939,
  COMPRESSED_TEXTURE_FORMATS: 34467,
  DONT_CARE: 4352,
  FASTEST: 4353,
  NICEST: 4354,
  GENERATE_MIPMAP_HINT: 33170,
  BYTE: 5120,
  UNSIGNED_BYTE: 5121,
  SHORT: 5122,
  UNSIGNED_SHORT: 5123,
  INT: 5124,
  UNSIGNED_INT: 5125,
  FLOAT: 5126,
  DEPTH_COMPONENT: 6402,
  ALPHA: 6406,
  RGB: 6407,
  RGBA: 6408,
  LUMINANCE: 6409,
  LUMINANCE_ALPHA: 6410,
  UNSIGNED_SHORT_4_4_4_4: 32819,
  UNSIGNED_SHORT_5_5_5_1: 32820,
  UNSIGNED_SHORT_5_6_5: 33635,
  FRAGMENT_SHADER: 35632,
  VERTEX_SHADER: 35633,
  MAX_VERTEX_ATTRIBS: 34921,
  MAX_VERTEX_UNIFORM_VECTORS: 36347,
  MAX_VARYING_VECTORS: 36348,
  MAX_COMBINED_TEXTURE_IMAGE_UNITS: 35661,
  MAX_VERTEX_TEXTURE_IMAGE_UNITS: 35660,
  MAX_TEXTURE_IMAGE_UNITS: 34930,
  MAX_FRAGMENT_UNIFORM_VECTORS: 36349,
  SHADER_TYPE: 35663,
  DELETE_STATUS: 35712,
  LINK_STATUS: 35714,
  VALIDATE_STATUS: 35715,
  ATTACHED_SHADERS: 35717,
  ACTIVE_UNIFORMS: 35718,
  ACTIVE_ATTRIBUTES: 35721,
  SHADING_LANGUAGE_VERSION: 35724,
  CURRENT_PROGRAM: 35725,
  NEVER: 512,
  LESS: 513,
  EQUAL: 514,
  LEQUAL: 515,
  GREATER: 516,
  NOTEQUAL: 517,
  GEQUAL: 518,
  ALWAYS: 519,
  KEEP: 7680,
  REPLACE: 7681,
  INCR: 7682,
  DECR: 7683,
  INVERT: 5386,
  INCR_WRAP: 34055,
  DECR_WRAP: 34056,
  VENDOR: 7936,
  RENDERER: 7937,
  VERSION: 7938,
  NEAREST: 9728,
  LINEAR: 9729,
  NEAREST_MIPMAP_NEAREST: 9984,
  LINEAR_MIPMAP_NEAREST: 9985,
  NEAREST_MIPMAP_LINEAR: 9986,
  LINEAR_MIPMAP_LINEAR: 9987,
  TEXTURE_MAG_FILTER: 10240,
  TEXTURE_MIN_FILTER: 10241,
  TEXTURE_WRAP_S: 10242,
  TEXTURE_WRAP_T: 10243,
  TEXTURE: 5890,
  TEXTURE_CUBE_MAP: 34067,
  TEXTURE_BINDING_CUBE_MAP: 34068,
  TEXTURE_CUBE_MAP_POSITIVE_X: 34069,
  TEXTURE_CUBE_MAP_NEGATIVE_X: 34070,
  TEXTURE_CUBE_MAP_POSITIVE_Y: 34071,
  TEXTURE_CUBE_MAP_NEGATIVE_Y: 34072,
  TEXTURE_CUBE_MAP_POSITIVE_Z: 34073,
  TEXTURE_CUBE_MAP_NEGATIVE_Z: 34074,
  MAX_CUBE_MAP_TEXTURE_SIZE: 34076,
  TEXTURE0: 33984,
  TEXTURE1: 33985,
  TEXTURE2: 33986,
  TEXTURE3: 33987,
  TEXTURE4: 33988,
  TEXTURE5: 33989,
  TEXTURE6: 33990,
  TEXTURE7: 33991,
  TEXTURE8: 33992,
  TEXTURE9: 33993,
  TEXTURE10: 33994,
  TEXTURE11: 33995,
  TEXTURE12: 33996,
  TEXTURE13: 33997,
  TEXTURE14: 33998,
  TEXTURE15: 33999,
  TEXTURE16: 34000,
  TEXTURE17: 34001,
  TEXTURE18: 34002,
  TEXTURE19: 34003,
  TEXTURE20: 34004,
  TEXTURE21: 34005,
  TEXTURE22: 34006,
  TEXTURE23: 34007,
  TEXTURE24: 34008,
  TEXTURE25: 34009,
  TEXTURE26: 34010,
  TEXTURE27: 34011,
  TEXTURE28: 34012,
  TEXTURE29: 34013,
  TEXTURE30: 34014,
  TEXTURE31: 34015,
  ACTIVE_TEXTURE: 34016,
  REPEAT: 10497,
  CLAMP_TO_EDGE: 33071,
  MIRRORED_REPEAT: 33648,
  FLOAT_VEC2: 35664,
  FLOAT_VEC3: 35665,
  FLOAT_VEC4: 35666,
  INT_VEC2: 35667,
  INT_VEC3: 35668,
  INT_VEC4: 35669,
  BOOL: 35670,
  BOOL_VEC2: 35671,
  BOOL_VEC3: 35672,
  BOOL_VEC4: 35673,
  FLOAT_MAT2: 35674,
  FLOAT_MAT3: 35675,
  FLOAT_MAT4: 35676,
  SAMPLER_2D: 35678,
  SAMPLER_CUBE: 35680,
  VERTEX_ATTRIB_ARRAY_ENABLED: 34338,
  VERTEX_ATTRIB_ARRAY_SIZE: 34339,
  VERTEX_ATTRIB_ARRAY_STRIDE: 34340,
  VERTEX_ATTRIB_ARRAY_TYPE: 34341,
  VERTEX_ATTRIB_ARRAY_NORMALIZED: 34922,
  VERTEX_ATTRIB_ARRAY_POINTER: 34373,
  VERTEX_ATTRIB_ARRAY_BUFFER_BINDING: 34975,
  IMPLEMENTATION_COLOR_READ_TYPE: 35738,
  IMPLEMENTATION_COLOR_READ_FORMAT: 35739,
  COMPILE_STATUS: 35713,
  LOW_FLOAT: 36336,
  MEDIUM_FLOAT: 36337,
  HIGH_FLOAT: 36338,
  LOW_INT: 36339,
  MEDIUM_INT: 36340,
  HIGH_INT: 36341,
  FRAMEBUFFER: 36160,
  RENDERBUFFER: 36161,
  RGBA4: 32854,
  RGB5_A1: 32855,
  RGB565: 36194,
  DEPTH_COMPONENT16: 33189,
  STENCIL_INDEX: 6401,
  STENCIL_INDEX8: 36168,
  DEPTH_STENCIL: 34041,
  RENDERBUFFER_WIDTH: 36162,
  RENDERBUFFER_HEIGHT: 36163,
  RENDERBUFFER_INTERNAL_FORMAT: 36164,
  RENDERBUFFER_RED_SIZE: 36176,
  RENDERBUFFER_GREEN_SIZE: 36177,
  RENDERBUFFER_BLUE_SIZE: 36178,
  RENDERBUFFER_ALPHA_SIZE: 36179,
  RENDERBUFFER_DEPTH_SIZE: 36180,
  RENDERBUFFER_STENCIL_SIZE: 36181,
  FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE: 36048,
  FRAMEBUFFER_ATTACHMENT_OBJECT_NAME: 36049,
  FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL: 36050,
  FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE: 36051,
  COLOR_ATTACHMENT0: 36064,
  DEPTH_ATTACHMENT: 36096,
  STENCIL_ATTACHMENT: 36128,
  DEPTH_STENCIL_ATTACHMENT: 33306,
  NONE: 0,
  FRAMEBUFFER_COMPLETE: 36053,
  FRAMEBUFFER_INCOMPLETE_ATTACHMENT: 36054,
  FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT: 36055,
  FRAMEBUFFER_INCOMPLETE_DIMENSIONS: 36057,
  FRAMEBUFFER_UNSUPPORTED: 36061,
  FRAMEBUFFER_BINDING: 36006,
  RENDERBUFFER_BINDING: 36007,
  MAX_RENDERBUFFER_SIZE: 34024,
  INVALID_FRAMEBUFFER_OPERATION: 1286,
  UNPACK_FLIP_Y_WEBGL: 37440,
  UNPACK_PREMULTIPLY_ALPHA_WEBGL: 37441,
  CONTEXT_LOST_WEBGL: 37442,
  UNPACK_COLORSPACE_CONVERSION_WEBGL: 37443,
  BROWSER_DEFAULT_WEBGL: 37444
};
var extensions = {
  // ratified
  OES_texture_float: {},
  OES_texture_half_float: {},
  WEBGL_lose_context: {
    loseContext: function loseContext() {}
  },
  OES_standard_derivatives: {},
  OES_vertex_array_object: {
    createVertexArrayOES: function createVertexArrayOES() {},
    bindVertexArrayOES: function bindVertexArrayOES() {},
    deleteVertexArrayOES: function deleteVertexArrayOES() {}
  },
  WEBGL_debug_renderer_info: null,
  WEBGL_debug_shaders: null,
  WEBGL_compressed_texture_s3tc: null,
  WEBGL_depth_texture: {},
  OES_element_index_uint: {},
  EXT_texture_filter_anisotropic: null,
  EXT_frag_depth: {},
  WEBGL_draw_buffers: {},
  ANGLE_instanced_arrays: null,
  OES_texture_float_linear: null,
  OES_texture_half_float_linear: null,
  EXT_blend_minmax: {
    MIN_EXT: 0,
    MAX_EXT: 0
  },
  EXT_shader_texture_lod: null,
  // community
  WEBGL_compressed_texture_atc: null,
  WEBGL_compressed_texture_pvrtc: null,
  EXT_color_buffer_half_float: null,
  WEBGL_color_buffer_float: null,
  EXT_sRGB: null,
  WEBGL_compressed_texture_etc1: null
};

var WebGLRenderingContext = function WebGLRenderingContext(canvas) {
  var _this = this;

  this.getShaderPrecisionFormat = function () {
    return {
      rangeMin: 127,
      rangeMax: 127,
      precision: 23
    };
  };

  this.GL_VERSION = 7938;
  this.SCISSOR_BOX = 3088;
  this.VIEWPORT = 2978;

  this.getParameter = function (paramId) {
    switch (paramId) {
      case _this.GL_VERSION:
        return ['WebGL1'];

      case _this.SCISSOR_BOX:
      case _this.VIEWPORT:
        return [0, 0, 1, 1];
    }
  };

  this.getExtension = function (ext) {
    return extensions[ext];
  };

  this.getProgramInfoLog = function () {
    return '';
  };

  this.getShaderInfoLog = function () {
    return '';
  };

  this.canvas = canvas;
  this.drawingBufferWidth = canvas.width;
  this.drawingBufferHeight = canvas.height;
  functions.forEach(function (func) {
    _this[func] = function () {
      return {};
    };
  });
  Object.keys(enums).forEach(function (key) {
    _this[key] = enums[key];
  });
};

var createWebGLContext = function createWebGLContext(canvas) {
  return new WebGLRenderingContext(canvas);
};

var createEventFirer = function createEventFirer(act, store) {
  var findEventHandler = function findEventHandler(element, eventName) {
    var eventHandlerName = toEventHandlerName(eventName);
    var props = element.props;

    if (typeof props[eventHandlerName] === 'function') {
      return props[eventHandlerName];
    }

    if (typeof props[eventName] === 'function') {
      return props[eventName];
    }

    console.warn("Handler for " + eventName + " was not found. You must pass event names in camelCase or name of the handler https://github.com/pmndrs/react-three-fiber/blob/master/packages/test-renderer/markdown/rttr.md#create-fireevent");
    return null;
  };

  var createSyntheticEvent = function createSyntheticEvent(element, data) {
    var raycastEvent = _extends({
      camera: store.getState().camera,
      stopPropagation: function stopPropagation() {},
      target: element,
      currentTarget: element,
      sourceEvent: data
    }, data);

    return raycastEvent;
  };

  var invokeEvent = /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(element, eventName, data) {
      var handler, returnValue;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              handler = findEventHandler(element, eventName);

              if (handler) {
                _context2.next = 3;
                break;
              }

              return _context2.abrupt("return");

            case 3:
              _context2.next = 5;
              return act( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        returnValue = handler(createSyntheticEvent(element, data));

                      case 1:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              })));

            case 5:
              return _context2.abrupt("return", returnValue);

            case 6:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function invokeEvent(_x, _x2, _x3) {
      return _ref.apply(this, arguments);
    };
  }();

  var fireEvent = /*#__PURE__*/function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(element, eventName, data) {
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              if (data === void 0) {
                data = {};
              }

              _context3.next = 3;
              return invokeEvent(element, eventName, data);

            case 3:
              return _context3.abrupt("return", _context3.sent);

            case 4:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    }));

    return function fireEvent(_x4, _x5, _x6) {
      return _ref3.apply(this, arguments);
    };
  }();

  return fireEvent;
};

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var expectOne = function expectOne(items, msg) {
  if (items.length === 1) {
    return items[0];
  }

  var prefix = items.length === 0 ? 'RTTR: No instances found' : "RTTR: Expected 1 but found " + items.length + " instances";
  throw new Error(prefix + " " + msg);
};
var matchProps = function matchProps(props, filter) {
  for (var key in filter) {
    if (props[key] !== filter[key]) {
      return false;
    }
  }

  return true;
};
var findAll = function findAll(root, decider) {
  var results = [];

  if (decider(root)) {
    results.push(root);
  }

  root.allChildren.forEach(function (child) {
    results.push.apply(results, findAll(child, decider));
  });
  return results;
};

var ReactThreeTestInstance = /*#__PURE__*/function () {
  function ReactThreeTestInstance(_fiber) {
    var _this = this;

    this.getChildren = function (fiber, opts) {
      if (opts === void 0) {
        opts = {
          exhaustive: false
        };
      }

      if (opts.exhaustive) {
        /**
         * this will return objects like
         * color or effects etc.
         */
        return [].concat((fiber.children || []).map(function (fib) {
          return wrapFiber(fib);
        }), fiber.__r3f.objects.map(function (fib) {
          return wrapFiber(fib);
        }));
      } else {
        return (fiber.children || []).map(function (fib) {
          return wrapFiber(fib);
        });
      }
    };

    this.find = function (decider) {
      return expectOne(findAll(_this, decider), "matching custom checker: " + decider.toString());
    };

    this.findAll = function (decider) {
      return findAll(_this, decider);
    };

    this.findByType = function (type) {
      return expectOne(findAll(_this, function (node) {
        return Boolean(node.type && node.type === type);
      }), "with node type: \"" + (type || 'Unknown') + "\"");
    };

    this.findAllByType = function (type) {
      return findAll(_this, function (node) {
        return Boolean(node.type && node.type === type);
      });
    };

    this.findByProps = function (props) {
      return expectOne(_this.findAllByProps(props), "with props: " + JSON.stringify(props));
    };

    this.findAllByProps = function (props) {
      return findAll(_this, function (node) {
        return Boolean(node.props && matchProps(node.props, props));
      });
    };

    this._fiber = _fiber;
  }

  _createClass(ReactThreeTestInstance, [{
    key: "instance",
    get: function get() {
      return this._fiber;
    }
  }, {
    key: "type",
    get: function get() {
      return this._fiber.type;
    }
  }, {
    key: "props",
    get: function get() {
      return this._fiber.__r3f.memoizedProps;
    }
  }, {
    key: "parent",
    get: function get() {
      var parent = this._fiber.parent;

      if (parent !== null) {
        return wrapFiber(parent);
      }

      return parent;
    }
  }, {
    key: "children",
    get: function get() {
      return this.getChildren(this._fiber);
    }
  }, {
    key: "allChildren",
    get: function get() {
      return this.getChildren(this._fiber, {
        exhaustive: true
      });
    }
  }]);

  return ReactThreeTestInstance;
}();
var fiberToWrapper = new WeakMap();
var wrapFiber = function wrapFiber(fiber) {
  var wrapper = fiberToWrapper.get(fiber);

  if (wrapper === undefined) {
    wrapper = new ReactThreeTestInstance(fiber);
    fiberToWrapper.set(fiber, wrapper);
  }

  return wrapper;
};

var index$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  ReactThreeTestInstance: ReactThreeTestInstance
});

var create = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(element, options) {
    var canvas, _fiber, scene, _store;

    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            canvas = createCanvas({
              width: options == null ? void 0 : options.width,
              height: options == null ? void 0 : options.height,
              beforeReturn: function beforeReturn(canvas) {
                //@ts-ignore
                canvas.getContext = function (type) {
                  if (type === 'webgl' || type === 'webgl2') {
                    return createWebGLContext(canvas);
                  }
                };
              }
            });
            _fiber = canvas;
            scene = null;
            _context7.next = 5;
            return fiber.reconciler.act( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
              return regeneratorRuntime.wrap(function _callee$(_context) {
                while (1) {
                  switch (_context.prev = _context.next) {
                    case 0:
                      scene = fiber.render(element, _fiber, _extends({
                        frameloop: 'never'
                      }, options, {
                        events: undefined
                      })).getState().scene;

                    case 1:
                    case "end":
                      return _context.stop();
                  }
                }
              }, _callee);
            })));

          case 5:
            _store = fiber._roots.get(_fiber).store;
            return _context7.abrupt("return", {
              scene: wrapFiber(scene),
              unmount: function () {
                var _unmount2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
                  return regeneratorRuntime.wrap(function _callee3$(_context3) {
                    while (1) {
                      switch (_context3.prev = _context3.next) {
                        case 0:
                          _context3.next = 2;
                          return fiber.reconciler.act( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
                            return regeneratorRuntime.wrap(function _callee2$(_context2) {
                              while (1) {
                                switch (_context2.prev = _context2.next) {
                                  case 0:
                                    fiber.unmountComponentAtNode(_fiber);

                                  case 1:
                                  case "end":
                                    return _context2.stop();
                                }
                              }
                            }, _callee2);
                          })));

                        case 2:
                        case "end":
                          return _context3.stop();
                      }
                    }
                  }, _callee3);
                }));

                function unmount() {
                  return _unmount2.apply(this, arguments);
                }

                return unmount;
              }(),
              getInstance: function getInstance() {
                var _mockRoots$get;

                // this is our root
                var fiber$1 = (_mockRoots$get = fiber._roots.get(_fiber)) == null ? void 0 : _mockRoots$get.fiber;
                var root = {
                  /**
                   * we wrap our child in a Provider component
                   * and context.Provider, so do a little
                   * artificial dive to get round this and
                   * pass context.Provider as if it was the
                   * actual react root
                   */
                  current: fiber$1.current.child.child
                };

                if (fiber$1.current.child.child) {
                  /**
                   * so this actually returns the instance
                   * the user has passed through as a Fiber
                   */
                  return fiber.reconciler.getPublicRootInstance(root);
                } else {
                  return null;
                }
              },
              update: function () {
                var _update = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(newElement) {
                  var _mockRoots$get2;

                  var fiber$1;
                  return regeneratorRuntime.wrap(function _callee5$(_context5) {
                    while (1) {
                      switch (_context5.prev = _context5.next) {
                        case 0:
                          fiber$1 = (_mockRoots$get2 = fiber._roots.get(_fiber)) == null ? void 0 : _mockRoots$get2.fiber;

                          if (!fiber$1) {
                            _context5.next = 4;
                            break;
                          }

                          _context5.next = 4;
                          return fiber.reconciler.act( /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
                            return regeneratorRuntime.wrap(function _callee4$(_context4) {
                              while (1) {
                                switch (_context4.prev = _context4.next) {
                                  case 0:
                                    fiber.reconciler.updateContainer(newElement, fiber$1, null, function () {
                                      return null;
                                    });

                                  case 1:
                                  case "end":
                                    return _context4.stop();
                                }
                              }
                            }, _callee4);
                          })));

                        case 4:
                          return _context5.abrupt("return");

                        case 5:
                        case "end":
                          return _context5.stop();
                      }
                    }
                  }, _callee5);
                }));

                function update(_x3) {
                  return _update.apply(this, arguments);
                }

                return update;
              }(),
              toTree: function toTree$1() {
                return toTree(scene);
              },
              toGraph: function toGraph$1() {
                return toGraph(scene);
              },
              fireEvent: createEventFirer(fiber.reconciler.act, _store),
              advanceFrames: function () {
                var _advanceFrames = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(frames, delta) {
                  var state, storeSubscribers, promises;
                  return regeneratorRuntime.wrap(function _callee6$(_context6) {
                    while (1) {
                      switch (_context6.prev = _context6.next) {
                        case 0:
                          if (delta === void 0) {
                            delta = 1;
                          }

                          state = _store.getState();
                          storeSubscribers = state.internal.subscribers;
                          promises = [];
                          storeSubscribers.forEach(function (subscriber) {
                            var _loop = function _loop(i) {
                              if (is.arr(delta)) {
                                promises.push(new Promise(function () {
                                  return subscriber.ref.current(state, delta[i] || delta[-1]);
                                }));
                              } else {
                                promises.push(new Promise(function () {
                                  return subscriber.ref.current(state, delta);
                                }));
                              }
                            };

                            for (var i = 0; i < frames; i++) {
                              _loop(i);
                            }
                          });
                          Promise.all(promises);

                        case 6:
                        case "end":
                          return _context6.stop();
                      }
                    }
                  }, _callee6);
                }));

                function advanceFrames(_x4, _x5) {
                  return _advanceFrames.apply(this, arguments);
                }

                return advanceFrames;
              }()
            });

          case 7:
          case "end":
            return _context7.stop();
        }
      }
    }, _callee7);
  }));

  return function create(_x, _x2) {
    return _ref.apply(this, arguments);
  };
}();

var act = fiber.act;
var index = {
  create: create,
  act: act
};

exports.ReactThreeTest = index$1;
exports.default = index;
